**********************************************************************************;
* create_datasetxml_standalone.sas                                               *;
*                                                                                *;
* Sample driver program to create Dataset-XML V1.0.0 files from a library of SAS *;
* data sets.                                                                     *;
*                                                                                *;
* Assumptions:                                                                   *;
* The code, as written, is designed to be run as stand-alone code, with the user *;
* responsible for all library assignments. All information will be in the LOG.   *;
* No results data set will be created.                                           *;
*                                                                                *;
* CST version  1.7                                                               *;
*                                                                                *;
* The following filename and libname statements may need to be changed by the    *;
* user to ensure the correct paths.                                              *;
**********************************************************************************;

**********************************************************************************;
* Location of the Dataset-XML root                                               *;
**********************************************************************************;
%let DatasetXMLRoot=C:/cstGlobalLibrary/standards/cdisc-datasetxml-1.0.0-1.7;

**********************************************************************************;
* Set Root paths for input and output                                            *;
**********************************************************************************;
%let studyRootPath=C:/cstSampleLibrary/cdisc-datasetxml-1.0.0-1.7;
%let studyOutputPath=C:/cstSampleLibrary/cdisc-datasetxml-1.0.0-1.7;

**********************************************************************************;
* Make the macros available                                                      *;
**********************************************************************************;
options  mautosource sasautos=("&DatasetXMLRoot/macros", sasautos);

************************************************************;
* Debugging aid:  set _cstDebug=1                          *;
************************************************************;
%let _cstDebug=0;
data _null_;
  _cstDebug = input(symget('_cstDebug'),8.);
  if _cstDebug then
    call execute("options mprint mlogic symbolgen mautolocdisplay;");
  else
    call execute("options nomprint nomlogic nosymbolgen nomautolocdisplay;");
run;

**********************************************************************************;
* Create SDTM Dataset-XML files                                                  *;
**********************************************************************************;
libname srcdata  "&studyRootPath/data";
libname xmldata  "&studyOutputPath/sourcexml";
filename srcmeta "&studyOutputPath/sourcexml/define.xml";

%datasetxml_write(
  _cstSourceLibrary=srcdata,
  _cstOutputLibrary=xmldata,
  _cstSourceMetadataDefineFileRef=srcmeta,
  _cstCheckLengths=Y,
  _cstIndent=N,
  _cstZip=Y,
  _cstDeleteAfterZip=N
  );  
  
libname srcdata clear;
libname xmldata clear;
filename srcmeta clear;

**********************************************************************************;
* Create ADaM Dataset-XML files                                                  *;
**********************************************************************************;
libname srcdata  "&studyRootPath/data_adam";
libname xmldata  "&studyOutputPath/sourcexml_adam";
filename srcmeta "&studyOutputPath/sourcexml_adam/define_adam.xml";

%datasetxml_write(
  _cstSourceLibrary=srcdata,
  _cstOutputLibrary=xmldata,
  _cstSourceMetadataDefineFileRef=srcmeta,
  _cstCheckLengths=Y,
  _cstIndent=N,
  _cstZip=Y,
  _cstDeleteAfterZip=N
  );
  
libname srcdata clear;
libname xmldata clear;
filename srcmeta clear;
